@@ -21,31 +21,31 @@ package org.apache.royale.compiler.internal.codegen.js.jx;
 
 import org.apache.royale.compiler.codegen.ISubEmitter;
 import org.apache.royale.compiler.codegen.js.IJSEmitter;
-import org.apache.royale.compiler.constants.IASLanguageConstants;
+import org.apache.royale.compiler.constants.IASLanguageConstants.BuiltinType;
 import org.apache.royale.compiler.definitions.IDefinition;
-import org.apache.royale.compiler.definitions.metadata.IMetaTag;
-import org.apache.royale.compiler.definitions.metadata.IMetaTagAttribute;
+import org.apache.royale.compiler.definitions.IFunctionDefinition;
+import org.apache.royale.compiler.definitions.IFunctionDefinition.FunctionClassification;
+import org.apache.royale.compiler.definitions.ITypeDefinition;
+import org.apache.royale.compiler.definitions.IVariableDefinition;
+import org.apache.royale.compiler.definitions.IVariableDefinition.VariableClassification;
 import org.apache.royale.compiler.internal.codegen.as.ASEmitterTokens;
 import org.apache.royale.compiler.internal.codegen.js.JSEmitterTokens;
 import org.apache.royale.compiler.internal.codegen.js.JSSubEmitter;
-import org.apache.royale.compiler.internal.codegen.js.royale.JSRoyaleDocEmitter;
 import org.apache.royale.compiler.internal.codegen.js.royale.JSRoyaleEmitter;
 import org.apache.royale.compiler.internal.codegen.js.royale.JSRoyaleEmitterTokens;
 import org.apache.royale.compiler.internal.codegen.js.goog.JSGoogEmitterTokens;
 import org.apache.royale.compiler.internal.definitions.AccessorDefinition;
-import org.apache.royale.compiler.internal.tree.as.DynamicAccessNode;
-import org.apache.royale.compiler.internal.tree.as.FunctionCallNode;
-import org.apache.royale.compiler.internal.tree.as.IdentifierNode;
-import org.apache.royale.compiler.internal.tree.as.MemberAccessExpressionNode;
-import org.apache.royale.compiler.internal.tree.as.UnaryOperatorAtNode;
+import org.apache.royale.compiler.internal.definitions.AppliedVectorDefinition;
+import org.apache.royale.compiler.internal.definitions.NamespaceDefinition;
+import org.apache.royale.compiler.internal.projects.RoyaleJSProject;
+import org.apache.royale.compiler.internal.semantics.SemanticUtils;
+import org.apache.royale.compiler.internal.tree.as.*;
 import org.apache.royale.compiler.tree.ASTNodeID;
-import org.apache.royale.compiler.tree.as.IASNode;
-import org.apache.royale.compiler.tree.as.IBinaryOperatorNode;
-import org.apache.royale.compiler.tree.as.IClassNode;
-import org.apache.royale.compiler.tree.as.IExpressionNode;
-import org.apache.royale.compiler.tree.as.IIdentifierNode;
+import org.apache.royale.compiler.tree.as.*;
 import org.apache.royale.compiler.utils.ASNodeUtils;
 
+import java.util.ArrayList;
+
 public class BinaryOperatorEmitter extends JSSubEmitter implements
         ISubEmitter<IBinaryOperatorNode>
 {
@@ -81,6 +81,9 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
         }
         else if (id == ASTNodeID.Op_InstanceOfID)
         {
+			if (ASNodeUtils.hasParenOpen(node)) {
+				write(ASEmitterTokens.PAREN_OPEN);
+			}
             getWalker().walk(node.getLeftOperandNode());
 
             startMapping(node, node.getLeftOperandNode());
@@ -90,11 +93,47 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
 
             IDefinition dnode = (node.getRightOperandNode())
                     .resolve(getProject());
-            if (dnode != null)
-                write(getEmitter()
-                        .formatQualifiedName(dnode.getQualifiedName()));
-            else
-                getWalker().walk(node.getRightOperandNode());
+			if (dnode != null)
+			{
+				String dnodeQname = dnode.getQualifiedName();
+                boolean isPackageOrFileMember = false;
+                if (dnode instanceof IVariableDefinition)
+                {
+                    IVariableDefinition variable = (IVariableDefinition) dnode;
+                    VariableClassification classification = variable.getVariableClassification();
+                    if (classification == VariableClassification.PACKAGE_MEMBER ||
+                            classification == VariableClassification.FILE_MEMBER)
+                    {
+                        isPackageOrFileMember = true;
+                    }
+                }
+                else if (dnode instanceof IFunctionDefinition)
+                {
+                    IFunctionDefinition func = (IFunctionDefinition) dnode;
+                    FunctionClassification classification = func.getFunctionClassification();
+                    if (classification == FunctionClassification.PACKAGE_MEMBER ||
+                            classification == FunctionClassification.FILE_MEMBER)
+                    {
+                        isPackageOrFileMember = true;
+                    }
+                }
+				else if(dnode instanceof ITypeDefinition)
+				{
+					isPackageOrFileMember = true;
+				}
+				if(isPackageOrFileMember)
+				{
+					dnodeQname = getEmitter().formatQualifiedName(dnodeQname);
+				}
+                write(dnodeQname);
+			}
+			else
+			{
+				getWalker().walk(node.getRightOperandNode());
+			}
+			if (ASNodeUtils.hasParenClose(node)) {
+				write(ASEmitterTokens.PAREN_CLOSE);
+			}
         }
         else
         {
@@ -104,9 +143,10 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
             {
                 IASNode lnode = leftSide.getChild(0);
                 IASNode rnode = leftSide.getChild(1);
-                IDefinition rnodeDef = (rnode instanceof IIdentifierNode) ? 
+                IDefinition rnodeDef = (rnode instanceof IIdentifierNode) ?
                 		((IIdentifierNode) rnode).resolve(getWalker().getProject()) :
                 		null;
+                boolean isDynamicAccess = rnode instanceof DynamicAccessNode;
                 if (lnode.getNodeID() == ASTNodeID.SuperID
                         && rnodeDef instanceof AccessorDefinition)
                 {
@@ -120,18 +160,17 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
                         else
                         	write(getEmitter().formatQualifiedName(
                         		getModel().getCurrentClass().getQualifiedName()));
-                        			
+                        		
                         write(ASEmitterTokens.MEMBER_ACCESS);
                         write(JSGoogEmitterTokens.SUPERCLASS);
                         write(ASEmitterTokens.MEMBER_ACCESS);
-                        write(JSRoyaleEmitterTokens.SETTER_PREFIX);
-                        write(rnodeDef.getBaseName());
+                        write(fjs.formatSetter(rnodeDef.getBaseName()));
                         write(ASEmitterTokens.MEMBER_ACCESS);
                         write(JSEmitterTokens.APPLY);
                         write(ASEmitterTokens.PAREN_OPEN);
                         write(ASEmitterTokens.THIS);
                         writeToken(ASEmitterTokens.COMMA);
-                        writeToken(ASEmitterTokens.SQUARE_OPEN);
+                        write(ASEmitterTokens.SQUARE_OPEN);
                         if (op.length() > 1) // += and things like that
                         {
                             write(getEmitter().formatQualifiedName(
@@ -139,8 +178,7 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
                             write(ASEmitterTokens.MEMBER_ACCESS);
                             write(JSGoogEmitterTokens.SUPERCLASS);
                             write(ASEmitterTokens.MEMBER_ACCESS);
-                            write(JSRoyaleEmitterTokens.GETTER_PREFIX);
-                            write(rnodeDef.getBaseName());
+							write(fjs.formatGetter(rnodeDef.getBaseName()));
                             write(ASEmitterTokens.MEMBER_ACCESS);
                             write(JSEmitterTokens.APPLY);
                             write(ASEmitterTokens.PAREN_OPEN);
@@ -150,29 +188,45 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
                         }
 
                         getWalker().walk(node.getRightOperandNode());
-                        writeToken(ASEmitterTokens.SQUARE_CLOSE);
+                        write(ASEmitterTokens.SQUARE_CLOSE);
                         write(ASEmitterTokens.PAREN_CLOSE);
                         return;
                     }
                 }
-                else if (((JSRoyaleEmitter)getEmitter()).isXMLList((MemberAccessExpressionNode)leftSide))
+                else if (((JSRoyaleEmitter)getEmitter()).isXMLList((IMemberAccessExpressionNode)leftSide))
                 {
                 	MemberAccessExpressionNode xmlNode = (MemberAccessExpressionNode)leftSide;
                 	if (node.getNodeID() == ASTNodeID.Op_AssignId)
                 	{
+                		boolean wrapQuotes = true;
 	                    getWalker().walk(xmlNode.getLeftOperandNode());
 	                    IExpressionNode rightSide = xmlNode.getRightOperandNode();
 	                    if (rightSide instanceof UnaryOperatorAtNode)
 	                    {
 		                    write(".setAttribute('");
-		                    getWalker().walk(((UnaryOperatorAtNode)rightSide).getChild(0));
+		                    getWalker().walk(rightSide.getChild(0));
 	                    }
+	                    else if (rightSide instanceof IDynamicAccessNode && ((IDynamicAccessNode) rightSide).getLeftOperandNode().getNodeID() == ASTNodeID.Op_AtID) {
+							write(".setAttribute(");
+							wrapQuotes = false;
+							getWalker().walk(((IDynamicAccessNode)rightSide).getRightOperandNode());
+						}
+	                    else if (rightSide instanceof INamespaceAccessExpressionNode) {
+							write(".setChild(");
+							write("new QName(");
+							getWalker().walk(((INamespaceAccessExpressionNode) rightSide).getLeftOperandNode());
+							write(",'");
+							getWalker().walk(((INamespaceAccessExpressionNode) rightSide).getRightOperandNode());
+							write("')");
+							wrapQuotes = false;
+						}
 	                    else
 	                    {
 		                    write(".setChild('");
 		                    getWalker().walk(rightSide);
 	                    }
-	                    write("', ");
+	                    if (wrapQuotes) write("'");
+	                    write(", ");
 	                    getWalker().walk(node.getRightOperandNode());
 	                    write(ASEmitterTokens.PAREN_CLOSE);
 	                    return;
@@ -208,7 +262,74 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
 	                    return;
                 	}
                 }
-                else if (((JSRoyaleEmitter)getEmitter()).isProxy((MemberAccessExpressionNode)leftSide))
+                else if (isDynamicAccess && ((JSRoyaleEmitter)getEmitter()).isXMLish((IExpressionNode)lnode))
+                {
+                	DynamicAccessNode dyn = (DynamicAccessNode)rnode;
+                	ITypeDefinition type = dyn.getRightOperandNode().resolveType(getProject());
+                	if (type.isInstanceOf("String", getProject()) || type.isInstanceOf("Object", getProject())
+                			|| type == getProject().getBuiltinType(BuiltinType.ANY_TYPE))
+        			{
+                		String field;
+                    	if (node.getNodeID() == ASTNodeID.Op_AssignId)
+                    	{
+    	                    getWalker().walk(lnode);
+    	                    IExpressionNode dynLeft = dyn.getLeftOperandNode();
+    	                    IExpressionNode dynRight = dyn.getRightOperandNode();
+    	                    if (dynLeft instanceof UnaryOperatorAtNode)
+    	                    {
+    		                    write(".setAttribute(");
+    							field = fjs.stringifyNode(dyn.getRightOperandNode());
+    	                    }
+    	                    else if (dynRight instanceof UnaryOperatorAtNode)
+    	                    {
+    		                    write(".setAttribute(");
+    							field = fjs.stringifyNode(dynRight.getChild(0));
+    	                    }
+    	                    else
+    	                    {
+    		                    write(".setChild(");
+    							field = fjs.stringifyNode(dynLeft);
+    	                    }
+    	                    write(field + ", ");
+    	                    getWalker().walk(node.getRightOperandNode());
+    	                    write(ASEmitterTokens.PAREN_CLOSE);
+    	                    return;
+                    	}
+                    	else if (node.getNodeID() == ASTNodeID.Op_AddAssignID)
+                    	{
+    	                    getWalker().walk(lnode);
+    	                    IExpressionNode rightSide = dyn.getRightOperandNode();
+    	                    if (rightSide instanceof UnaryOperatorAtNode)
+    	                    {
+    		                    write(".setAttribute('");
+    							field = fjs.stringifyNode(((UnaryOperatorAtNode)rightSide).getChild(0));
+    							field = field.replace("\"", ""); // remove wrapping double-quotes
+    	                    }
+    	                    else
+    	                    {
+    		                    write(".setChild('");
+    							field = fjs.stringifyNode(rightSide);
+    							field = field.replace("\"", ""); // remove wrapping double-quotes
+    	                    }
+    	                    write(field + "', ");
+                            getWalker().walk(node.getLeftOperandNode());
+    	                    write(".plus(");
+    	                    getWalker().walk(node.getRightOperandNode());
+    	                    write(ASEmitterTokens.PAREN_CLOSE);
+    	                    write(ASEmitterTokens.PAREN_CLOSE);
+    	                    return;
+                    	}
+                    	else if (node.getNodeID() == ASTNodeID.Op_AddID)
+                    	{
+    	                    getWalker().walk(dyn);
+    	                    write(".plus(");
+    	                    getWalker().walk(node.getRightOperandNode());
+    	                    write(ASEmitterTokens.PAREN_CLOSE);
+    	                    return;
+                    	}
+        			}
+                }
+                else if (((JSRoyaleEmitter)getEmitter()).isProxy(((MemberAccessExpressionNode)leftSide).getLeftOperandNode()) && leftDef == null)
                 {
                 	MemberAccessExpressionNode proxyNode = (MemberAccessExpressionNode)leftSide;
                 	if (node.getNodeID() == ASTNodeID.Op_AssignId)
@@ -250,7 +371,7 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
             else if (leftSide.getNodeID() == ASTNodeID.IdentifierID)
             {
     			if ((leftDef != null)
-    				&& IdentifierNode.isXMLish(leftDef, getWalker().getProject()))
+    				&& SemanticUtils.isXMLish(leftDef, getWalker().getProject()))
     			{
                 	if (node.getNodeID() == ASTNodeID.Op_AddAssignID)
                 	{
@@ -264,161 +385,154 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
                 	}
                 }
             }
+            else if (leftSide.getNodeID() == ASTNodeID.ArrayIndexExpressionID) // dynamic access
+            {
+            	DynamicAccessNode dyn = (DynamicAccessNode)leftSide;
+            	IExpressionNode dynLeft = dyn.getLeftOperandNode();
+            	ITypeDefinition type = dyn.getRightOperandNode().resolveType(getProject());
+            	if (((JSRoyaleEmitter)getEmitter()).isXMLish(dynLeft)/* && !SemanticUtils.isNumericType(type, getProject())*/) //type.isInstanceOf("String", getProject())
+    			{
+            		String field;
+                	if (node.getNodeID() == ASTNodeID.Op_AssignId)
+                	{
+	                    getWalker().walk(dynLeft);
+	                    IExpressionNode rightSide = dyn.getRightOperandNode();
+	                    if (rightSide instanceof UnaryOperatorAtNode)
+	                    {
+		                    write(".setAttribute(");
+							field = fjs.stringifyNode((rightSide).getChild(0));
+	                    }
+	                    else
+	                    {
+							write(".setChild(");
+							field = fjs.stringifyNode(rightSide);
+	                    }
+						if (field.startsWith("\"") && field.endsWith("\"")) {
+							// remove wrapping double-quotes and swap to single quotes
+							field = "'" + field.substring(1, field.length() - 1) + "'";
+						}
+	                    write(field + ", ");
+	                    getWalker().walk(node.getRightOperandNode());
+	                    write(ASEmitterTokens.PAREN_CLOSE);
+	                    return;
+                	}
+                	else if (node.getNodeID() == ASTNodeID.Op_AddAssignID)
+                	{
+	                    getWalker().walk(dynLeft);
+	                    IExpressionNode rightSide = dyn.getRightOperandNode();
+	                    if (rightSide instanceof UnaryOperatorAtNode)
+	                    {
+		                    write(".setAttribute('");
+							field = fjs.stringifyNode(((UnaryOperatorAtNode)rightSide).getChild(0));
+							field = field.replace("\"", ""); // remove wrapping double-quotes
+	                    }
+	                    else
+	                    {
+		                    write(".setChild('");
+							field = fjs.stringifyNode(rightSide);
+							field = field.replace("\"", ""); // remove wrapping double-quotes
+	                    }
+	                    write(field + "', ");
+                        getWalker().walk(node.getLeftOperandNode());
+	                    write(".plus(");
+	                    getWalker().walk(node.getRightOperandNode());
+	                    write(ASEmitterTokens.PAREN_CLOSE);
+	                    write(ASEmitterTokens.PAREN_CLOSE);
+	                    return;
+                	}
+                	else if (node.getNodeID() == ASTNodeID.Op_AddID)
+                	{
+	                    getWalker().walk(dyn);
+	                    write(".plus(");
+	                    getWalker().walk(node.getRightOperandNode());
+	                    write(ASEmitterTokens.PAREN_CLOSE);
+	                    return;
+                	}
 
-            boolean leftIsNumber = (leftDef != null && (leftDef.getQualifiedName().equals(IASLanguageConstants.Number) ||
-					  leftDef.getQualifiedName().equals(IASLanguageConstants._int) ||
-					  leftDef.getQualifiedName().equals(IASLanguageConstants.uint)));
-        	IExpressionNode rNode = node.getRightOperandNode();
-        	IDefinition rightDef = rNode.resolveType(getWalker().getProject());
-        	boolean rightIsNumber = (rightDef != null && (rightDef.getQualifiedName().equals(IASLanguageConstants.Number) ||
-					  rightDef.getQualifiedName().equals(IASLanguageConstants._int) ||
-					  rightDef.getQualifiedName().equals(IASLanguageConstants.uint)));
-            if (leftIsNumber && !rightIsNumber && (rightDef == null || rightDef.getQualifiedName().equals(IASLanguageConstants.ANY_TYPE)))
-            {
-        		if (rNode.getNodeID() == ASTNodeID.FunctionCallID)
-        		{
-	            	IExpressionNode fnNameNode = ((FunctionCallNode)rNode).getNameNode();
-	            	if (fnNameNode.getNodeID() == ASTNodeID.MemberAccessExpressionID)
-	            	{
-	            		MemberAccessExpressionNode mae = (MemberAccessExpressionNode)fnNameNode;
-	            		IExpressionNode rightNode = mae.getRightOperandNode();
-	            		rightIsNumber = rightNode.getNodeID() == ASTNodeID.IdentifierID && 
-	            				((IdentifierNode)rightNode).getName().equals("length") &&
-	            				fjs.isXMLList(mae);
-	            	}
-        		}
-        		else if (rNode.getNodeID() == ASTNodeID.ArrayIndexExpressionID)
-        		{
-        			DynamicAccessNode dyn = (DynamicAccessNode)rNode;
-        			IDefinition lDef = dyn.getLeftOperandNode().resolveType(getProject());
-        			IDefinition rDef = dyn.getRightOperandNode().resolveType(getProject());
-        			// numeric indexing?
-        			if (rDef.getQualifiedName().equals(IASLanguageConstants.Number))
-        			{
-        				IMetaTag[] metas = lDef.getAllMetaTags();
-        				for (IMetaTag meta : metas)
-        				{
-        					if (meta.getTagName().equals("ArrayElementType"))
-        					{
-        						IMetaTagAttribute[] attrs = meta.getAllAttributes();
-        						for (IMetaTagAttribute attr : attrs)
-        						{
-        							String t = attr.getValue();
-            						if (t.equals(IASLanguageConstants.Number))
-            							rightIsNumber = true;
-        						}
-        					}
-        				}
-        			}
-        		}
-            }
-            String coercion = (leftIsNumber && !rightIsNumber && isAssignment) ? "Number(" : "";
-            if (isAssignment && leftDef != null && leftDef.getQualifiedName().equals(IASLanguageConstants.String))
-            {
-            	if (rNode.getNodeID() != ASTNodeID.LiteralStringID &&
-            			rNode.getNodeID() != ASTNodeID.LiteralNullID)
+    			}
+            	else if (((JSRoyaleEmitter)getEmitter()).isProxy(dynLeft))
             	{
-		        	if (rightDef == null ||
-		        			(!(rightDef.getQualifiedName().equals(IASLanguageConstants.String) ||
-		        			  (rightDef.getQualifiedName().equals(IASLanguageConstants.ANY_TYPE)
-		                    		&& rNode.getNodeID() == ASTNodeID.FunctionCallID &&
-		                    		isToString(rNode)) ||
-		        			  // if not an assignment we don't need to coerce numbers
-		        			  (!isAssignment && rightIsNumber) ||
-		        			   rightDef.getQualifiedName().equals(IASLanguageConstants.Null))))
-		        	{
-		        		JSRoyaleDocEmitter docEmitter = (JSRoyaleDocEmitter)(getEmitter().getDocEmitter());
-		        		if (docEmitter.emitStringConversions)
-		        		{
-		        			coercion = "org.apache.royale.utils.Language.string(";
-		        		}
-		        	}
+            		if (isAssignment)
+            		{
+	                    getWalker().walk(dynLeft);
+	                    IExpressionNode rightSide = dyn.getRightOperandNode();
+	                    write(".setProperty(");
+	                    getWalker().walk(rightSide);
+	                    write(", ");
+	                    getWalker().walk(node.getRightOperandNode());
+	                    write(ASEmitterTokens.PAREN_CLOSE);
+            		}
+            		else
+            		{
+	                    getWalker().walk(dynLeft);
+	                    IExpressionNode rightSide = dyn.getRightOperandNode();
+	                    write(".getProperty(");
+	                    getWalker().walk(rightSide);
+	                    write(ASEmitterTokens.PAREN_CLOSE);
+	                    write(ASEmitterTokens.SPACE);
+	                    writeToken(op);
+	                    getWalker().walk(node.getRightOperandNode());
+            		}
+                    return;            		
             	}
             }
-            super_emitBinaryOperator(node, coercion);
-            if (coercion.length() > 0)
-            	write(")");
-            	
-            /*
-            IExpressionNode leftSide = node.getLeftOperandNode();
+            
+			if (id == ASTNodeID.Op_EqualID || id ==ASTNodeID.Op_NotEqualID) {
+				//QName == QName
+				if (leftDef != null && leftDef.getQualifiedName().equals("QName")) {
+					IDefinition rightDef = node.getRightOperandNode().resolveType(getProject());
+					if (rightDef != null && rightDef.getQualifiedName().equals("QName")) {
+						//handle non-strict equality/inequality a little differently
+						if (id == ASTNodeID.Op_NotEqualID) write("!");
+						write("QName.equality(");
+						getWalker().walk(node.getLeftOperandNode());
+						write(",");
+						getWalker().walk(node.getRightOperandNode());
+						write(")");
+						return;
+					}
+				} else if (leftDef != null && getProject().getBuiltinType(BuiltinType.BOOLEAN).equals(leftDef) && SemanticUtils.isXMLish(node.getRightOperandNode(), getProject())) {
+					boolean literalBool = node.getLeftOperandNode().getNodeID() == ASTNodeID.LiteralBooleanID;
+					//note, this only covers boolean ==/!= xmlish, not: xmlish ==/!= boolean
+					if (literalBool) {
+						write("'");
+					}
+					else write("('' + ");
+						getWalker().walk(node.getLeftOperandNode());
+					if (literalBool) {
+						write("'");
+					}
+					else write(")");
+					write(" " + node.getOperator().getOperatorText() + " ");
 
-            IExpressionNode property = null;
-            int leftSideChildCount = leftSide.getChildCount();
-            if (leftSideChildCount > 0)
-            {
-                IASNode childNode = leftSide.getChild(leftSideChildCount - 1);
-                if (childNode instanceof IExpressionNode)
-                    property = (IExpressionNode) childNode;
-                else
-                    property = leftSide;
-            }
-            else
-                property = leftSide;
+					getWalker().walk(node.getRightOperandNode());
+					return;
+				} else if ((((leftDef == null || getProject().getBuiltinType(BuiltinType.ANY_TYPE).equals(leftDef)) && SemanticUtils.isXMLish(node.getLeftOperandNode(), getProject())) || SemanticUtils.isXMLish(leftDef, getProject())) && getProject().getBuiltinType(BuiltinType.BOOLEAN).equals(node.getRightOperandNode().resolveType(getProject()))) {
+					boolean literalBool = node.getRightOperandNode().getNodeID() == ASTNodeID.LiteralBooleanID;
 
-            IDefinition def = null;
-            if (property instanceof IIdentifierNode)
-                def = ((IIdentifierNode) property).resolve(getWalker()
-                        .getProject());
-
-            boolean isSuper = false;
-            if (leftSide.getNodeID() == ASTNodeID.MemberAccessExpressionID)
-            {
-                IASNode cnode = leftSide.getChild(0);
-                ASTNodeID cId = cnode.getNodeID();
-
-                isSuper = cId == ASTNodeID.SuperID;
-            }
-
-            String op = node.getOperator().getOperatorText();
-            boolean isAssignment = op.contains("=") && !op.contains("==") && 
-                                                    !(op.startsWith("<") || 
-                                                            op.startsWith(">") || 
-                                                            op.startsWith("!"));
-
-            if (def instanceof AccessorDefinition && isAssignment)
-            {
-                // this will make the set_foo call
-                getWalker().walk(leftSide);
-            }
-            else if (isSuper) 
-            {
-                emitSuperCall(node, "");
-            }
-            else
-            {
-                if (ASNodeUtils.hasParenOpen(node))
-                    write(ASEmitterTokens.PAREN_OPEN);
-
-                getWalker().walk(leftSide);
-
-                if (node.getNodeID() != ASTNodeID.Op_CommaID)
-                    write(ASEmitterTokens.SPACE);
-
-                writeToken(node.getOperator().getOperatorText());
-
-                getWalker().walk(node.getRightOperandNode());
-
-                if (ASNodeUtils.hasParenClose(node))
-                    write(ASEmitterTokens.PAREN_CLOSE);
-            }
-            */
+					//note, this only covers xmlish ==/!= boolean, not: boolean ==/!= xmlish
+					getWalker().walk(node.getLeftOperandNode());
+					write(" " + node.getOperator().getOperatorText() + " ");
+					if (literalBool) {
+						write("'");
+					}
+					else write("('' + ");
+					getWalker().walk(node.getRightOperandNode());
+					if (literalBool) {
+						write("'");
+					}
+					else write(")");
+					return;
+				}
+			}
+			
+		
+			super_emitBinaryOperator(node, isAssignment);
         }
     }
     
-    private boolean isToString(IASNode rNode)
-    {
-    	IExpressionNode fnNameNode = ((FunctionCallNode)rNode).getNameNode();
-    	if (fnNameNode.getNodeID() == ASTNodeID.MemberAccessExpressionID)
-    	{
-    		MemberAccessExpressionNode mae = (MemberAccessExpressionNode)fnNameNode;
-    		IExpressionNode rightNode = mae.getRightOperandNode();
-    		return rightNode.getNodeID() == ASTNodeID.IdentifierID && 
-    				((IdentifierNode)rightNode).getName().equals("toString");
-    	}
-    	return false;
-    }
 
-    private void super_emitBinaryOperator(IBinaryOperatorNode node, String coercion)
+    private void super_emitBinaryOperator(IBinaryOperatorNode node, boolean isAssignment)
     {
         if (ASNodeUtils.hasParenOpen(node))
             write(ASEmitterTokens.PAREN_OPEN);
@@ -453,26 +567,52 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
         }
         else
         {
-            getWalker().walk(node.getLeftOperandNode());
-
+			if (isAssignment
+					&& (getProject() instanceof RoyaleJSProject && ((RoyaleJSProject) getProject()).config != null && ((RoyaleJSProject) getProject()).config.getJsVectorEmulationClass() == null)
+					&& node.getLeftOperandNode() instanceof MemberAccessExpressionNode
+					&& ((MemberAccessExpressionNode) node.getLeftOperandNode()).getRightOperandNode() instanceof IdentifierNode
+					&& ((IdentifierNode) ((MemberAccessExpressionNode) node.getLeftOperandNode()).getRightOperandNode()).getName().equals("length")
+					&& ((MemberAccessExpressionNode) node.getLeftOperandNode()).getLeftOperandNode().resolveType(getProject()) instanceof AppliedVectorDefinition)
+			{
+				//for default Vector implementation, when setting length, we need to set it on the associated 'synthType' instance which tags the native
+				//Array representation of the Vector. This allows running 'setter' code because it is not possible to override the native length setter on Array
+				//unless using a different approach, like es6 Proxy.
+				//this code inserts the extra access name for setting length, e.g. myVectInstance['_synthType'].length = assignedValue
+				//the dynamic access field name is a constant on Language, so it can be different/shorter in release build
+				getWalker().walk(((MemberAccessExpressionNode) node.getLeftOperandNode()).getLeftOperandNode());
+				write(ASEmitterTokens.SQUARE_OPEN);
+				write(JSRoyaleEmitterTokens.LANGUAGE_QNAME.getToken());
+				write(ASEmitterTokens.MEMBER_ACCESS);
+				write(JSRoyaleEmitterTokens.ROYALE_SYNTH_TAG_FIELD_NAME);
+				write(ASEmitterTokens.SQUARE_CLOSE);
+				write(ASEmitterTokens.MEMBER_ACCESS);
+				getWalker().walk(((MemberAccessExpressionNode) node.getLeftOperandNode()).getRightOperandNode());
+			}
+			else if (isAssignment && node.getLeftOperandNode() instanceof NamespaceAccessExpressionNode)
+			{
+				getWalker().walk(node.getLeftOperandNode().getChild(1));
+			}
+            else getWalker().walk(node.getLeftOperandNode());
             startMapping(node, node.getLeftOperandNode());
-            
-            if (id != ASTNodeID.Op_CommaID)
+			boolean xmlAdd = false;
+            if (id == ASTNodeID.Op_AddID && SemanticUtils.isXMLish(node.getLeftOperandNode(),getProject()) && SemanticUtils.isXMLish(node.getRightOperandNode(),getProject())) {
+				//we need to use 'plus' method instead of '+'
+				xmlAdd = true;
+			}
+            if (id != ASTNodeID.Op_CommaID && !xmlAdd)
                 write(ASEmitterTokens.SPACE);
 
             // (erikdebruin) rewrite 'a &&= b' to 'a = a && b'
             if (id == ASTNodeID.Op_LogicalAndAssignID
                     || id == ASTNodeID.Op_LogicalOrAssignID)
             {
-                IIdentifierNode lnode = (IIdentifierNode) node
+                IExpressionNode lnode = node
                         .getLeftOperandNode();
 
                 writeToken(ASEmitterTokens.EQUAL);
                 endMapping(node);
 
-                startMapping(node);
-                write(lnode.getName());
-                endMapping(node);
+                getWalker().walk(lnode);
 
                 startMapping(node, node.getLeftOperandNode());
                 write(ASEmitterTokens.SPACE);
@@ -481,31 +621,40 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
             }
             else
             {
-                write(node.getOperator().getOperatorText());
+				if (xmlAdd) {
+					write(".plus(");
+				} else {
+					write(node.getOperator().getOperatorText());
+				}
             }
 
             write(ASEmitterTokens.SPACE);
             endMapping(node);
 
-            write(coercion);
-            /*
-            IDefinition definition = node.getRightOperandNode().resolve(getProject());
-        	if (definition instanceof FunctionDefinition &&
-        			(!(definition instanceof AccessorDefinition)))
-        	{
-        	}
-        	else */
-        		getWalker().walk(node.getRightOperandNode());
-                if (node.getNodeID() == ASTNodeID.Op_InID &&
-                        ((JSRoyaleEmitter)getEmitter()).isXML(node.getRightOperandNode()))
-                {
-                	write(".elementNames()");
-                }   
-                else if (node.getNodeID() == ASTNodeID.Op_InID &&
-                        ((JSRoyaleEmitter)getEmitter()).isProxy(node.getRightOperandNode()))
-                {
-                	write(".propertyNames()");
-                }
+			if (isAssignment)
+			{
+				getEmitter().emitAssignmentCoercion(node.getRightOperandNode(), node.getLeftOperandNode().resolveType(getProject()));
+			}
+			else
+			{
+				
+				getWalker().walk(node.getRightOperandNode());
+				
+				if (node.getNodeID() == ASTNodeID.Op_InID &&
+						((JSRoyaleEmitter)getEmitter()).isXMLish(node.getRightOperandNode()))
+				{
+					write(".elementNames()");
+				}
+				else if (node.getNodeID() == ASTNodeID.Op_InID &&
+						((JSRoyaleEmitter)getEmitter()).isProxy(node.getRightOperandNode()))
+				{
+					//@todo: add loop target null safety (see changes in ForEachEmitter)
+					write(".propertyNames()");
+				}
+			}
+			if (xmlAdd) {
+				write(")");
+			}
         }
 
         if (ASNodeUtils.hasParenOpen(node))
@@ -518,9 +667,11 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
     	FULLYEAR("fullYear", "getFullYear"),
     	MONTH("month", "getMonth"),
     	DATE("date", "getDate"),
+    	DAY("day", "getDay"),
     	FULLYEARUTC("fullYearUTC", "getUTCFullYear"),
     	MONTHUTC("monthUTC", "getUTCMonth"),
     	DATEUTC("dateUTC", "getUTCDate"),
+    	DAYUTC("dayUTC", "getUTCDay"),
     	HOURS("hours", "getHours"),
     	MINUTES("minutes", "getMinutes"),
     	SECONDS("seconds", "getSeconds"),
@@ -528,7 +679,8 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
     	HOURSUTC("hoursUTC", "getUTCHours"),
     	MINUTESUTC("minutesUTC", "getUTCMinutes"),
     	SECONDSUTC("secondsUTC", "getUTCSeconds"),
-    	MILLISECONDSUTC("millisecondsUTC", "getUTCMilliseconds");
+    	MILLISECONDSUTC("millisecondsUTC", "getUTCMilliseconds"),
+    	TIMEZONEOFFSET("timezoneOffset", "getTimezoneOffset");
     	
     	DatePropertiesGetters(String value, String functionName)
     	{
@@ -556,9 +708,11 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
     	FULLYEAR("fullYear", "setFullYear"),
     	MONTH("month", "setMonth"),
     	DATE("date", "setDate"),
+    	DAY("day", "setDay"),
     	FULLYEARUTC("fullYearUTC", "setUTCFullYear"),
     	MONTHUTC("monthUTC", "setUTCMonth"),
     	DATEUTC("dateUTC", "setUTCDate"),
+    	DAYUTC("day", "setUTCDay"),
     	HOURS("hours", "setHours"),
     	MINUTES("minutes", "setMinutes"),
     	SECONDS("seconds", "setSeconds"),
@@ -590,6 +744,9 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
     
     void specialCaseDate(IBinaryOperatorNode node, MemberAccessExpressionNode leftSide)
     {
+        if (ASNodeUtils.hasParenOpen(node))
+            write(ASEmitterTokens.PAREN_OPEN);
+
     	MemberAccessExpressionNode dateNode = (MemberAccessExpressionNode)leftSide;
         IIdentifierNode rightSide = (IIdentifierNode)dateNode.getRightOperandNode();
         String op = node.getOperator().getOperatorText();
@@ -632,5 +789,8 @@ public class BinaryOperatorEmitter extends JSSubEmitter implements
         	write(ASEmitterTokens.SPACE);
 	        getWalker().walk(node.getRightOperandNode());
         }
+
+        if (ASNodeUtils.hasParenOpen(node))
+            write(ASEmitterTokens.PAREN_CLOSE);
     }
 }
